@@ -1,6 +1,7 @@
 # The COPYRIGHT file at the top level of this repository contains the full
 # copyright notices and license terms.
 import copy
+
 from decimal import Decimal
 from time import *
 
@@ -858,74 +859,67 @@ class BatchLine(ModelSQL, ModelView):
                 # Taxes with negative rates (e.g. intracommunity): like
                 # expenses
                 tax_amount_second_currency = None
-                abs_tax_amount = abs(tax_amount)
-                abs_base_amount = abs(base_amount)
-                abs_total_amount = abs(total_amount)
                 tax_credit_account = tax.invoice_account
                 tax_debit_account = tax.credit_note_account
-                if not cancel:
-                    if side == 'credit':
-                        tax_debit, tax_credit = -tax_amount, _ZERO
-                        tax_account = tax_credit_account
-                    elif side == 'debit':
-                        tax_debit, tax_credit = _ZERO, tax_amount
-                        tax_account = tax_debit_account
-                elif cancel:
-                    abs_tax_amount = -abs_tax_amount
-                    abs_base_amount = -abs_base_amount
-                    abs_total_amount = -abs_total_amount
-                    if side == 'debit':
-                        tax_debit, tax_credit = -tax_amount, _ZERO
-                        tax_account = tax_debit_account
-                    elif side == 'credit':
-                        tax_debit, tax_credit = _ZERO, tax_amount
-                        tax_account = tax_credit_account
 
+                def get_tax_group(tax):
+                    tax_group = tax.group
+                    if not tax_group and tax.parent:
+                        tax_group = get_tax_group(tax.parent)
+                    return tax_group
+
+                tax_group = get_tax_group(tax)
+                tax_group_kind = tax_group.kind if tax_group else None
+                if side == 'credit':
+                    tax_debit, tax_credit = -tax_amount, _ZERO
+                    tax_account = tax_credit_account
+                elif side == 'debit':
+                    tax_debit, tax_credit = _ZERO, tax_amount
+                    tax_account = tax_debit_account
+
+                tax_base_amount = base_amount
+                tax_tax_amount = tax_amount
+                if cancel:
+                    tax_debit, tax_credit = -tax_debit, -tax_credit
+                    tax_base_amount = -tax_base_amount
+                    tax_tax_amount = -tax_tax_amount
+
+                # Create the move line for the tax(es)
                 tax_move_line = self._get_move_line(
                     tax_debit, tax_credit, tax_account, second_currency,
                     tax_amount_second_currency)
 
+                if tax_group_kind == 'purchase':
+                    tax_tax_amount = -tax_tax_amount
+                    tax_base_amount = -tax_base_amount
+
                 # Create the tax_line(s) for the tax(es)
-                tax_line = self._get_tax_line(abs_tax_amount, 'tax', tax)
+                tax_line = self._get_tax_line(tax_tax_amount, 'tax', tax)
                 tax_move_line.tax_lines += (tax_line,)
 
                 # Create the tax_line(s) for the base
-                # Only one base line for all tax lines
-                if not base_tax_lines:
-                    base_tax_line = self._get_tax_line(abs_base_amount,
-                        'base', tax)
-                    base_tax_lines.append(base_tax_line)
+                base_tax_line = self._get_tax_line(tax_base_amount,
+                    'base', tax)
+                base_tax_lines.append(base_tax_line)
                 res.append(tax_move_line)
 
             # Create the move line(s) for the base and total
-            if not cancel:
-                if side == 'credit':
-                    base_debit, base_credit = -base_amount, _ZERO
+            if side == 'credit':
+                base_debit, base_credit = -base_amount, _ZERO
+                base_account = debit_account
+                total_debit, total_credit = _ZERO, -total_amount
+                total_account = credit_account
+            elif side == 'debit':
+                base_debit, base_credit = _ZERO, base_amount
+                base_account = credit_account
+                total_debit, total_credit = total_amount, _ZERO
+                total_account = debit_account
+                if reverse_charge:
                     base_account = debit_account
-                    total_debit, total_credit = _ZERO, -total_amount
                     total_account = credit_account
-                elif side == 'debit':
-                    base_debit, base_credit = _ZERO, base_amount
-                    base_account = credit_account
-                    total_debit, total_credit = total_amount, _ZERO
-                    total_account = debit_account
-                    if reverse_charge:
-                        base_account = debit_account
-                        total_account = credit_account
-            elif cancel:
-                if side == 'debit':
-                    base_debit, base_credit = -base_amount, _ZERO
-                    base_account = credit_account
-                    total_debit, total_credit = _ZERO, -total_amount
-                    total_account = debit_account
-                elif side == 'credit':
-                    base_debit, base_credit = _ZERO, base_amount
-                    base_account = debit_account
-                    total_debit, total_credit = total_amount, _ZERO
-                    total_account = credit_account
-                    if reverse_charge:
-                        base_account = credit_account
-                        total_account = debit_account
+            if cancel:
+                base_debit, base_credit = -base_debit, -base_credit
+                total_debit, total_credit = -total_debit, -total_credit
             base_move_line = self._get_move_line(
                 base_debit, base_credit, base_account, second_currency,
                 amount_second_currency)
@@ -938,18 +932,15 @@ class BatchLine(ModelSQL, ModelView):
 
         else:
             amount = abs(amount)
-            if not cancel:
-                res.append(self._get_move_line(_ZERO, amount, credit_account,
-                        second_currency, amount_second_currency))
-                res.append(self._get_move_line(amount, _ZERO, debit_account,
-                        second_currency, amount_second_currency))
-            else:
+            if amount_second_currency:
+                amount_second_currency = abs(amount_second_currency)
+            if cancel:
                 amount = -amount
                 if amount_second_currency:
                     amount_second_currency = -amount_second_currency
-                res.append(self._get_move_line(amount, _ZERO, credit_account,
+            res.append(self._get_move_line(_ZERO, amount, credit_account,
                         second_currency, amount_second_currency))
-                res.append(self._get_move_line(_ZERO, amount, debit_account,
+            res.append(self._get_move_line(amount, _ZERO, debit_account,
                         second_currency, amount_second_currency))
         return res
 
@@ -979,11 +970,13 @@ class BatchLine(ModelSQL, ModelView):
         context = Transaction().context
         date = self.date or context.get('date') or Date.today()
         res = []
-        if (tax.type != 'none'
-                and ((not tax.start_date or tax.start_date <= date)
-                and (not tax.end_date or tax.end_date >= date))):
-            res.append(tax)
-        if tax.childs:
+        tax_valid = False
+        if ((not tax.start_date or tax.start_date <= date)
+                and (not tax.end_date or tax.end_date >= date)):
+            tax_valid = True
+            if tax.type != 'none':
+                res.append(tax)
+        if tax_valid and tax.childs:
             for child in tax.childs:
                 res.extend(self._get_taxes(child))
         return res
@@ -1442,9 +1435,9 @@ class CancelBatchLines(Wizard):
             'batch': batch_line.batch,
             'account': batch_line.account,
             'contra_account': batch_line.contra_account,
-            'amount': batch_line.amount * -1,
+            'amount': batch_line.amount,
             'tax': batch_line.tax,
-            'is_cancelation_move': not bool(batch_line.is_cancelation_move),
+            'is_cancelation_move': True,
             'journal': batch_line.journal,
             'date': posting_date,
             'reference': reference,
